
#include "CSimNeut.hh"
// #include "CSimulationTool.hh"

#include "TROOT.h"
#include "TStyle.h"
#include "TFile.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TF1.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"
#include "TGraph2DErrors.h"
#include "TVector3.h"

#include <iostream>
#include <string>
#include <sstream>
#include <fstream>
#include <cmath>
#include <vector>
#include <deque>

#include "CKinema.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"

#include "TSimulationDataBeam.hh"
#include "TSimulationDataNeutronDetector.hh"
#include "TSimulationParameterNeutronDetector.hh"

ClassImp(CSimNeut)

CSimNeut::CSimNeut()
{
  fProton = new CParticle(CParticleDefinition::GetParticle("proton"));
  fNeutron = new CParticle(CParticleDefinition::GetParticle("neutron"));
  fChargedParticle = new CParticle(CParticleDefinition::GetParticle("9Li"));
  //fChargedParticle = new CParticle(CParticleDefinition::GetParticle("10Be"));
}

CSimNeut::~CSimNeut()
{
  delete fChargedParticle;
  delete fNeutron;
  delete fProton;
}

int CSimNeut::StartAnalysis(std::string rootfile)
{
  fTopDirectory = gDirectory;

  fFileSim = TFile::Open(rootfile.c_str(), "READ");
  if(!fFileSim) return 0;

  //  fFileExp = TFile::Open("root/exp.root","recreate");
  //  if(!fFileExp) return;

  fTopDirectory->cd();

  ReadParameters();

  CreateHist();

  CreateExpData();

  //  AnalysisExpData();

  //  fFileExp->Write();
  //  fFileExp->Close();
  fFileSim->Close();

  return 1;
}

int CSimNeut::ReadParameters()
{
  TSimulationParameterNeutronDetector* simulationParameterNeutronDetector;
  fFileSim->GetObject("parameterNeutronDetector", simulationParameterNeutronDetector);

  fSimPara = *simulationParameterNeutronDetector;
//  fModuleNumber = simulationParameterNeutronDetector.fModuleNumber;
//  fLayerNumber = simulationParameterNeutronDetector.fLayerNumber;
//  fStuckNumber = simulationParameterNeutronDetector.fStuckNumber;
//  fDetectorPos = simulationParameterNeutronDetector.fDetectorPos;
//  fModuleSize = simulationParameterNeutronDetector.fModuleSize;
//  fModuleGap = simulationParameterNeutronDetector.fModuleGap;
//  fLayerGap = simulationParameterNeutronDetector.fLayerGap;
//  fStuckGap = simulationParameterNeutronDetector.fStuckGap;
//  fRotationMode = simulationParameterNeutronDetector.fRotationMode;

  fTotalModuleNumber = fSimPara.fModuleNumber*fSimPara.fLayerNumber*fSimPara.fStuckNumber;

  fHardwareThreshold = 2; // MeV
  fSoftwareThreshold = 6; // MeV
  fTimeResolution = 0.3; // ns
  fAttenuationLength = 700; // cm
  fPropagationVelocity = 15.8; // cm/ns
  fMultiplicity = 1;

  fAnalysisMethod = "single";
  //  fAnalysisMethod = "track";

  fStandardKineticEenrgy = 250; // standard incident energy

  fNumberOfEntries = -1;
}

int CSimNeut::CreateHist()
{
  fHIDQL = new TH2D("hidql", "id vs qu", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, 0, 150);
  fHIDQR = new TH2D("hidqr", "id vs qd", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, 0, 150);
  fHIDTL = new TH2D("hidtl", "id vs tu", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, 0, 150);
  fHIDTR = new TH2D("hidtr", "id vs td", fTotalModuleNumber, 0.5, fTotalModuleNumber + 0.5, 300, 0, 150);

  fHTQ.resize(fTotalModuleNumber, (TH1*)0);
  for(int i=0; i<fTotalModuleNumber+1; ++i){
    {
      std::ostringstream name,title;
      name << "htq" << i;
      title << "t vs q id" << i;
      fHTQ[i] = new TH2D(name.str().c_str(), title.str().c_str(), 300,0,150, 300,0,150);
    }
  }
}

int CSimNeut::CreateExpData()
{
  TTree* tree;
  fFileSim->GetObject("simTree", tree);

  std::vector<TSimulationDataNeutronDetector>* simulationDataNeutronDetectorArray = new std::vector<TSimulationDataNeutronDetector>;
  TBranch* branchSimulationDataNeutronDetectorArray = 0;
  if(tree->SetBranchAddress("neutronDetector",&simulationDataNeutronDetectorArray,&branchSimulationDataNeutronDetectorArray) < 0) return 0;

  if(fNumberOfEntries < 0) fNumberOfEntries = tree->GetEntriesFast();
  for(int i=0; i<fNumberOfEntries; i++) {
    if((i+1)%10000 == 0) std::cout << "\r converted events:" << i+1 << std::flush;

    Long64_t entry = tree->LoadTree(i);
    branchSimulationDataNeutronDetectorArray->GetEntry(entry);

    if(simulationDataNeutronDetectorArray->size() == 0) continue;

    fID.clear();
    fQL.clear();
    fQR.clear();
    fTL.clear();
    fTR.clear();

    SimToExp(simulationDataNeutronDetectorArray);

    for(int j=0; j<(int)fID.size(); ++j){
      Int_t id = fID[j];
      Double_t ql = fQL[j];
      Double_t qr = fQR[j];
      Double_t tl = fTL[j];
      Double_t tr = fTR[j];

      //      std::cout << tl << " " << tr << " " << (tl+tr)/2 << " " << -fSimPara.fModuleSize.X()/fPropagationVelocity/2 << std::endl;

      fHIDQL->Fill(id, ql);
      fHIDQR->Fill(id, qr);
      fHIDTL->Fill(id, tl);
      fHIDTR->Fill(id, tr);
      fHTQ[0]->Fill((tl+tr)/2-fSimPara.fModuleSize.X()/fPropagationVelocity/2, sqrt(ql*qr));
      fHTQ[id]->Fill((tl+tr)/2-fSimPara.fModuleSize.X()/fPropagationVelocity/2, sqrt(ql*qr));
    }
  }

  tree->ResetBranchAddresses();

  return 1;
}

void CSimNeut::SimToExp(std::vector<TSimulationDataNeutronDetector>* simulationDataNeutronDetectorArray)
{
  for(int i=0; i<simulationDataNeutronDetectorArray->size(); ++i){
    TSimulationDataNeutronDetector simulationDataNeutronDetector = (*simulationDataNeutronDetectorArray)[i];
    std::vector<Int_t> id = simulationDataNeutronDetector.fModuleID;
    Int_t sequentialID = 1+id[0]+id[1]*fSimPara.fModuleNumber+id[2]*fSimPara.fModuleNumber*fSimPara.fLayerNumber;
    //    std::cout << sequentialID << " " << id[0] << " " << id[1] << " " << id[2] << std::endl;

    Double_t xl = fSimPara.fModuleSize.X()/2.
      -(simulationDataNeutronDetector.fPosition.X()-fSimPara.fDetectorPos.X());
    Double_t xr = fSimPara.fModuleSize.X()/2.
      +(simulationDataNeutronDetector.fPosition.X()-fSimPara.fDetectorPos.X());
    Double_t yl = fSimPara.fModuleSize.X()/2.
      -(simulationDataNeutronDetector.fPosition.Y()-fSimPara.fDetectorPos.Y()); // left == up
    Double_t yr = fSimPara.fModuleSize.X()/2.
      +(simulationDataNeutronDetector.fPosition.Y()-fSimPara.fDetectorPos.Y()); // right == down

    Double_t energyDepositEE = CKinema::MeVtoMeVee(simulationDataNeutronDetector.fKineticEnergy,
						   simulationDataNeutronDetector.fKineticEnergy
						   - simulationDataNeutronDetector.fEnergyDeposit,
						   simulationDataNeutronDetector.fParticleName.Data());

    Double_t tl, tr, ql, qr;
    if(fSimPara.fRotationMode == 1){ // stuck horizontal
      tl = simulationDataNeutronDetector.fTime + yl/fPropagationVelocity;
      tr = simulationDataNeutronDetector.fTime + yr/fPropagationVelocity;
      ql = energyDepositEE*exp(-yl/fAttenuationLength);
      qr = energyDepositEE*exp(-yr/fAttenuationLength);
    }else if(fSimPara.fRotationMode == 2){ // vertical, horizontal, vertical, ....
      if(id[0]%2){
	tl = simulationDataNeutronDetector.fTime + yl/fPropagationVelocity;
	tr = simulationDataNeutronDetector.fTime + yr/fPropagationVelocity;
	ql = energyDepositEE*exp(-yl/fAttenuationLength);
	qr = energyDepositEE*exp(-yr/fAttenuationLength);
      }else{
	tl = simulationDataNeutronDetector.fTime + xl/fPropagationVelocity;
	tr = simulationDataNeutronDetector.fTime + xr/fPropagationVelocity;
	ql = energyDepositEE*exp(-xl/fAttenuationLength);
	qr = energyDepositEE*exp(-xr/fAttenuationLength);
      }
    }else{ // stuck vertical
      tl = simulationDataNeutronDetector.fTime + xl/fPropagationVelocity;
      tr = simulationDataNeutronDetector.fTime + xr/fPropagationVelocity;
      ql = energyDepositEE*exp(-xl/fAttenuationLength);
      qr = energyDepositEE*exp(-xr/fAttenuationLength);
    }      

    bool flagFound = false;
    for(int j=0; j<(int)fID.size(); ++j){
      if(fID[j] == sequentialID){
	flagFound = true;
	if(fTL[j] > tl) fTL[j] = tl;
	if(fTR[j] > tr) fTR[j] = tr;
	fQL[j] += ql;
	fQR[j] += qr;
	break;
      }
    }
    if(!flagFound){
      fID.push_back(sequentialID);
      fTL.push_back(tl);
      fTR.push_back(tr);
      fQL.push_back(ql);
      fQR.push_back(qr);
    }
  }

  bool underThreshold = false;
  for(int i=0; i<(int)fID.size(); ++i){
    if(fQL[i] < fHardwareThreshold || fQR[i] < fHardwareThreshold){
      fID.erase(fID.begin()+i);
      fTL.erase(fTL.begin()+i);
      fTR.erase(fTR.begin()+i);
      fQL.erase(fQL.begin()+i);
      fQR.erase(fQR.begin()+i);
      --i;
      continue;
    }
  }

  for(int i=0; i<(int)fID.size(); ++i){
    fTL[i] += gRandom->Gaus(0,fTimeResolution);
    fTR[i] += gRandom->Gaus(0,fTimeResolution);  
  }
    
  // hitmintime
  Double_t tmin = 10000;
  while((int)fID.size() > 1){
    Double_t tmin = (fTL[0]+fTR[0])/2;
    Double_t t = (fTL[1]+fTR[1])/2;

    if(tmin > t){
      tmin = t;
      fID.erase(fID.begin());
      fTL.erase(fTL.begin());
      fTR.erase(fTR.begin());
      fQL.erase(fQL.begin());
      fQR.erase(fQR.begin());
    }else{
      fID.erase(fID.begin()+1);
      fTL.erase(fTL.begin()+1);
      fTR.erase(fTR.begin()+1);
      fQL.erase(fQL.begin()+1);
      fQR.erase(fQR.begin()+1);
    }
  }

//    Double_t x, y, z;
//    if(fRotationMode == 1){
//      if(it->fId[0]%2){
//	x = it->fId[1]*width + width/2. - fPropagationVelocityength/2. + shiftx;
//	y = fPropagationVelocity*(it->fTL-it->fTR)/2. + shifty;
//      }else{
//	x = fPropagationVelocity*(it->fTL-it->fTR)/2. + shiftx;
//	y = it->fId[1]*width + width/2. - fPropagationVelocityength/2. + shifty;
//      }
//    }else{
//      x = fPropagationVelocity*(it->fTL-it->fTR)/2. + shiftx;
//      y = it->fId[1]*width + width/2. - fPropagationVelocityength/2. + shifty;
//    }
//    z = it->fId[0]*(depth+gap) + zdepth + depth/2.;
//
//    it->fX.SetXYZ(x, y, z);
//
//    ++it;
//  }
//
//  if(method == "single"){
//    Single(dataexp);
//    //  }else if(method == "track"){
//    //    Track(dataexp);
//  }else{
//    std::cerr << "method:" << method << " cannot use" << std::endl;
//    return;
//  }
}

// void CSimNeut::Single(CDataHime* dataexp)
// {
//   static std::deque<CDataHime::OneData>::iterator it;
//   static std::deque<CDataHime::OneData>::iterator itmin;
// 
//   if(dataexp->fDequeOfOneData.empty()){
//     dataexp->fEventFlag = 0;
//     return;
//   }
// 
//   itmin = dataexp->fDequeOfOneData.begin();
//   it = dataexp->fDequeOfOneData.begin();
//   while(it != dataexp->fDequeOfOneData.end()){
//     if(itmin->fX.Z() > it->fX.Z()) itmin = it;
//     ++it;
//   }
// 
//   dataexp->fX.SetXYZT(itmin->fX.X(), itmin->fX.Y(), itmin->fX.Z(), itmin->fT);
// 
//   dataexp->CalMomentum();
// 
//   dataexp->fEventFlag = 1;
// 
//   return;
// }

// void CSimNeut::Track(CDataHime* dataexp)
// {
// #ifdef VISUALIZE
//   TGraph2DErrors* graph = new TGraph2DErrors;//staticを付けるとメモリの再確保がなくなり早い
// #endif
// 
//   if(dataexp->fDequeOfOneData.empty()) return;
// 
//   //どうせ何回も使うので
//   static std::deque<CDataHime::OneData>::iterator it;
//   static std::deque<CDataHime::OneData>::iterator itmin;
// 
//   //最速イベントではなく先頭イベント。
//   //最速を選ぶと時間分解能の分だけ微妙にシフトすると思われる。
//   itmin = dataexp->fDequeOfOneData.begin();
//   it = dataexp->fDequeOfOneData.begin();
//   while(it != dataexp->fDequeOfOneData.end()){
//     //    if(it->fX.Z() < itmin->fX.Z()) itmin = it;
//     if(it->fX.Z() == itmin->fX.Z()){
//       if(it->fT < itmin->fT) itmin = it;
//     }else if(it->fX.Z() < itmin->fX.Z()){
//       itmin = it;
//     }
//     ++it;
//   }
// 
//   //遅いイベントは関係ないので切り捨て。
//   it = dataexp->fDequeOfOneData.begin();
//   double tmin = itmin->fT;//roop内部でeraseを使ってるので途中でitminが不正になりうる。
//   while(it != dataexp->fDequeOfOneData.end()){
//     //先頭でhitしてからある程度時間がかかったeventは切ってしまう
//     //時間で排除する代わりに、シグナルの近傍度みたいな量を定義して、
//     //それでcutしたほうがノイズを効率的に落とせるかも
//     if(abs(it->fT - tmin) > 1.2){
//       it = dataexp->fDequeOfOneData.erase(it);
//       continue;
//     }
//     ++it;
//   }
// 
//   //eraseしたので先頭イベントのイテレータを取り直す。(必須!!)
//   itmin = dataexp->fDequeOfOneData.begin();
//   it = dataexp->fDequeOfOneData.begin();
//   while(it != dataexp->fDequeOfOneData.end()){
//     if(it->fX.Z() < itmin->fX.Z()) itmin = it;
//     //    if(it->fT < itmin->fT) itmin = it;
//     ++it;
//   }
// 
//   //hit数に満たない場合はcut
//   if((int)dataexp->fDequeOfOneData.size() < multihit){
//     dataexp->fEventFlag = 0;//in-validなデータの場合は0
//     return;
//   }
// 
//   //2Dfit(zの誤差を無視する)でtracking
//   Int_t point = 0;
//   std::vector<double> pointx, pointy, pointz;
//   std::vector<double> pointxe, pointye, pointze;
//   it = dataexp->fDequeOfOneData.begin();
//   while(it != dataexp->fDequeOfOneData.end()){
//     pointx.push_back(it->fX.X());
//     pointy.push_back(it->fX.Y());
//     pointz.push_back(it->fX.Z());
//     if(it->fId[0]%2){
//       pointxe.push_back(width/sqrt(12));
//       pointye.push_back(vl*treso/sqrt(2));
//     }else{	 
//       pointxe.push_back(vl*treso/sqrt(2));
//       pointye.push_back(width/sqrt(12));
//     }
//     pointze.push_back(depth/sqrt(12));
// 
// #ifdef VISUALIZE
//     graph->SetPoint(point, it->fX.X(), it->fX.Y(), it->fX.Z());
//     if(it->fId[0]%2){
//       graph->SetPointError(point, width/sqrt(12), vl*treso/sqrt(2), depth/sqrt(12));
//     }else{	 
//       graph->SetPointError(point, vl*treso/sqrt(2), width/sqrt(12), depth/sqrt(12));
//     }
// #endif
// 
//     ++point;
//     ++it;
//   }
//   double parFit[4];
// 
//   CAnalysis::LinearFit(pointz, pointx, pointxe, parFit[0], parFit[1]);
//   CAnalysis::LinearFit(pointz, pointy, pointye, parFit[2], parFit[3]);
//   double aminx = CAnalysis::SD(pointz, pointx, parFit[0], parFit[1]);
//   double aminy = CAnalysis::SD(pointz, pointy, parFit[2], parFit[3]);
//   double amin = sqrt(aminx*aminx+aminy*aminy);//rmsっぽいもの
// 
//   //rmsっぽいものを収集
//   hamin->Fill(amin);
// 
//   //理想的には選択的にトラックを引くアルゴリズムが必要。
//   //たとえばamin<5になるまで、計算に入れる点を変えるとか。
//   //今はaminminで単純に切り捨てごめん。
//   if(amin > aminmin){
//     dataexp->fEventFlag = 0;//in-validなデータの場合は0
//     return;
//   }
// 
//   //analysis of timing
//   TVector3 xf(parFit[1], parFit[3], 1.);//fitした直線にそったベクトル
//   double theta = xf.Angle(itmin->fX);
//   double theta_rp = xf.Theta();
//   double phi_rp = xf.Phi();
// 
//   // derive recoile proton energy from theta(lab)
//   CParticle pin = CParticleDefinition::GetParticle("proton");
//   CParticle nin = CParticleDefinition::GetParticle("neutron");
//   CParticle pout = CParticleDefinition::GetParticle("proton");
//   CParticle nout = CParticleDefinition::GetParticle("neutron");
// 
//   nin.k(ein-sp);//一陽子分離エネルギーを考慮。ほとんど効かないけど。
//   pin.k(0);//Fermi motionは無視して静止していると考える。
// 
//   //lab系のthetaをパラメータとして、剛体の2体散乱をさせる。
//   Double_t theta_cm = CKinema::Scatter(theta, 0., nin, pin, pout, nout);
// 
//   //pout,noutの角度を修正(上記のthetaはlab系でのneutronとの角度の差)
//   pout.thetaphi(theta_rp, phi_rp);
//   nout.thetaphibar(theta_rp, phi_rp);
// 
//   //recoil protonの情報の保存
//   dataexp->fPrp.SetPxPyPzE(pout.p(0), pout.p(1), pout.p(2), pout.e());
// 
//   //thetaを取得
//   htheta_exp->Fill(theta);
//   htheta_cm_exp->Fill(theta_cm);
// 
//   //timingの解析の本体
//   double timing = 0;
//   int count = 0;
//   it = dataexp->fDequeOfOneData.begin();
//   while(it != dataexp->fDequeOfOneData.end()){
//     double tsum = -1.;
//     if(it == itmin){
//       tsum = 0;
//       //先頭でhitしてから4層通過分程度時間がかかったeventは時間の計算には参加させない
//     }else if(abs(it->fT - itmin->fT) < 0.8){
//       double xtemp, ytemp, ztemp;
//       if(it->fId[0]-itmin->fId[0] > 0){
// 	//二層め以降で成ったシグナルは荷電粒子によるものと思われ、
// 	//シミュレーション的にはシンチレータの端面で鳴るのでZを補正。
// 	//実験でも基本的に同じ補正でいいはずだが未知
// 	CAnalysis::line(it->fX.Z()-depth/2., parFit, xtemp, ytemp, ztemp);
//       }else{
// 	CAnalysis::line(it->fX.Z(), parFit, xtemp, ytemp, ztemp);	
//       }
//       TVector3 X(xtemp, ytemp, ztemp);
//       CAnalysis::line(itmin->fX.Z(), parFit, xtemp, ytemp, ztemp);
//       TVector3 Xmin(xtemp, ytemp, ztemp);
//       TVector3 distance_v = X - Xmin;
//       double distance = distance_v.Mag();
// 
//       CParticle ptemp = pout;
// 
//       //検出器中を飛んだ時間の計算。先に物質中を飛んだ時間をだして、
//       //あとから残りの空間を飛んだ時間を足す。
//       //かなり荒いけど結構まともな近似
//       //      tsum = CKinema::BetheBloch::TimeOut
//       //	("p", "Scint", ptemp,
//       //	 distance - (it->fId[0]-itmin->fId[0])*gap/cos(distance_v.Theta()), step);
//       //      if(tsum >= 0.){
//       //	//ほんとは2cm+2cm+2cm+....と交互に時間を足していかなければならない。
//       //	//ここで少しmeanのずれが生じていると考えられる。
//       //	tsum += (distance/2.)/((pout.beta()+ptemp.beta())*CKinema::fLightVelocity/2.);
//       //      }
// 
//       //こっちの計算は真面目に計算した場合。エネルギーでのカットオフが必要
//       double flightcos = cos(distance_v.Theta());
//       int flightcount = 0;
//       double flightdistance;
// 
//       tsum = 0;//初期値はtsum=-1なので0にしておく
// 
//       while(1){
// 	if((flightcount % 2) == 0){
// 	  if(flightcount == 0){
// 	    flightdistance = depth/2/flightcos;//一枚目は真ん中で発生したと考える
// 	  }else{
// 	    flightdistance = depth/flightcos;
// 	  }
// 	}else{
// 	  flightdistance = gap/flightcos;	    
// 	}
// 
// 	if(distance < flightdistance) flightdistance = distance;
// 
// 	if((flightcount % 2) == 0){
// 	  double tsumtemp;
// 	  tsumtemp = CKinema::BetheBloch::TimeOut("p", "Scint", ptemp, flightdistance, step);
// 	  //止まってしまった場合
// 	  if(tsumtemp < 0){
// 	    tsum = -1;
// 	    break;
// 	  }
// 	  tsum += tsumtemp;
// 	}else{
// 	  tsum += flightdistance/(ptemp.beta()*CKinema::fLightVelocity);
// 	}
// 
// 	//ひょろひょろだったらその点を使った計算は不確定なのでいらない
// 	if(ptemp.k() < 20){
// 	  tsum = -1;
// 	  break;
// 	}
// 
// 	distance -= flightdistance;
// 	if(distance <= 0) break;//発光点まで来たので終了
// 
// 	++flightcount;
//       }
//     }
//     if(tsum >= 0){
//       //補正した結果、改善する方向に向かわない場合は補正に参加させない
//       if(abs((it->fT - tsum) - itmin->fT) < treso*2){
// 	timing += it->fT - tsum;
// 	++count;
//       }
//     }
//     ++it;
//   }
//   timing /= count;
// 
//   //位置はlinearfitで求まった先頭、時間は上の解析で補正した時間
//   double xtemp, ytemp, ztemp;
//   CAnalysis::line(itmin->fX.Z(),parFit,xtemp,ytemp,ztemp);
//   dataexp->fX.SetXYZT(xtemp, ytemp, ztemp, timing);
// 
//   //X,Y,Z,Tが求まったらMomentumを計算しておく。
//   //この関数の内部で非数が渡されてないかエラーチェックが走る
//   dataexp->CalMomentum();
// 
// #ifdef VISUALIZE
//   graph->SetMarkerStyle(8);
//   graph->SetMarkerSize(1);
//   graph->GetXaxis()->SetLimits(-length/2., length/2.);
//   graph->GetYaxis()->SetLimits(-length/2., length/2.);
//   graph->SetMinimum(zdepth);
//   graph->SetMaximum(zdepth+zdepth_back);
// 
//   if(0){
//     graph->Draw("PERR");
//   }else{
//     static bool init = true;
//     if(init){
//       graph->Draw("PERR");
//       init = false;
//     }else{
//       graph->Draw("PERRSAME");
//     }
//   }
// 
//   CAnalysis::DrawLine(parFit, kRed, zdepth, zdepth_back);
// 
//   TCanvas* canvas = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("canvas");
//   canvas->Update();
// 
//   CAnalysis::EventLoop();
// #endif
// 
//   dataexp->fEventFlag = 1;//validなデータの場合は1
// 
//   return;
// }

