
#include "TROOT.h"
#include "TStyle.h"
#include "TFile.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TH3D.h"
#include "TGraph2DErrors.h"
#include "TVector3.h"
#include "TF1.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"

#include <iostream>
#include <string>
#include <cmath>
#include <deque>
#include <fstream>

#include "CDataHime.hh"
#include "CDataHimeSim.hh"
#include "CKinema.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"
#include "FFunction.hh"
#include "CMapFunction1D.hh"
#include "CFold.hh"
#include "CAnalysis.hh"
#include "CXmlManager.hh"

#include "CAnalyzeTool.hh"
#include "CAnalyzeNeutGamma.hh"

ClassImp(CAnalyzeNeutGamma)

CAnalyzeNeutGamma::CAnalyzeNeutGamma(std::string rootfile)
{
  canvas = new TCanvas("canvas","",200,10,900,900);
  canvas->SetFillColor(0);

  topdir = gDirectory;

  filesim = TFile::Open(rootfile.c_str(), "READ");
  if(!filesim) abort();

  fileexp = TFile::Open("root/exp.root","recreate");
  if(!fileexp) abort();

  proton = new CParticle(CParticleDefinition::GetParticle("proton"));
  neutron = new CParticle(CParticleDefinition::GetParticle("neutron"));
}

CAnalyzeNeutGamma::~CAnalyzeNeutGamma()
{
  delete neutron;
  delete proton;

  fileexp->Write();
  fileexp->Close();
  filesim->Close();
}

void CAnalyzeNeutGamma::Main(std::string prmfile, std::string detector, std::string method, Long64_t nentries)
{
  if(!ReadParamaters(prmfile, detector)){
    std::cerr << "error occur during reading paramaters" << std::endl;
    return;
  }

  if(!CreateHist()){
    std::cerr << "error occur during creating hist" << std::endl;
    return;
  }    

  if(!CreateExpData(method, nentries)){
    std::cerr << "error occur during creating experiment data" << std::endl;
    return;
  }    

  if(!AnalysisExpData(nentries)){
    std::cerr << "error occur during analyzing experiment data" << std::endl;
    return;
  }    

  if(!DrawHist(nentries)){
    std::cerr << "error occur during drawing histogram" << std::endl;
    return;
  }    
}

int CAnalyzeNeutGamma::ReadParamaters(std::string prmfile, std::string detector)
{
  CXmlManager xml;
  if (!xml.OpenFile(prmfile)) return 0;

  if (!xml.GetValue("//"+detector+"//thrmethod", &thrmethod)) return 0;
  if (!xml.GetValue("//"+detector+"//thr", &thr)) return 0;
  if (!xml.GetValue("//"+detector+"//treso", &treso)) return 0;
  if (!xml.GetValue("//"+detector+"//attlength", &attlength)) return 0;
  if (!xml.GetValue("//"+detector+"//vl", &vl)) return 0;
  if (!xml.GetValue("//"+detector+"//rotation", &rotation)) return 0;
  if (!xml.GetValue("//"+detector+"//length", &length)) return 0;
  if (!xml.GetValue("//"+detector+"//vlength", &vlength)) return 0;
  if (!xml.GetValue("//"+detector+"//width", &width)) return 0;
  if (!xml.GetValue("//"+detector+"//depth", &depth)) return 0;
  if (!xml.GetValue("//"+detector+"//gap", &gap)) return 0;
  if (!xml.GetValue("//"+detector+"//zdepth", &zdepth)) return 0;
  if (!xml.GetValue("//"+detector+"//zdepth_back", &zdepth_back)) return 0;
  if (!xml.GetValue("//"+detector+"//shiftx", &shiftx)) return 0;
  if (!xml.GetValue("//"+detector+"//shifty", &shifty)) return 0;
  if (!xml.GetValue("//"+detector+"//multihit", &multihit)) return 0;
  if (!xml.GetValue("//"+detector+"//aminmin", &aminmin)) return 0;
  if (!xml.GetValue("//"+detector+"//ein", &ein)) return 0;
  if (!xml.GetValue("//"+detector+"//sp", &sp)) return 0;
  if (!xml.GetValue("//"+detector+"//step", &step)) return 0;

  return 1;
}

int CAnalyzeNeutGamma::CreateHist()
{
  topdir->cd();

  hamin = new TH1D("rms", "title", 100, 0., 20.);
  htheta_exp = new TH1D("theta_exp", "title", 100, 0., 3.14159);
  htheta_cm_exp = new TH1D("theta_cm_exp", "title", 100, 0., 3.14159);

  htheta_lab = new TH1D("theta_lab", "title", 100, -2., 2);
  hphi_lab = new TH1D("phi_lab", "title", 100, -2., 2);

  hthetaenergy_lab
    = new TH2D("thetaenergy_lab", "title", 100, 0., 3.14159, 100, 0., ein+20);
  hthetaenergy_lab_gate
    = new TH2D("thetaenergy_lab_gate", "title", 100, 0., 3.14159, 100, 0., ein+20);
  hthetaenergy_cm
    = new TH2D("thetaenergy_cm", "title", 100, 0., 3.14159, 100, 0., ein+20);
  hthetaenergy_cm_gate
    = new TH2D("thetaenergy_cm_gate", "title", 100, 0., 3.14159, 100, 0., ein+20);

  hdee_lab
    = new TH2D("dee_lab", "title", 100, 0., 100, 100, 0., ein+20);
  hthetade_lab
    = new TH2D("thetade_lab", "title", 100, 0., 3.14159, 100, 0., 100);
  hthetadee_lab
    = new TH3D("thetadee_lab", "title",
	       100, 0., 3.14159,
	       100, 0., 100,
	       100, 0., ein+20);

  htheta_cm = new TH1D("theta_cm", "title", 100, 0., 3.14159);
  htheta_cm_gate = new TH1D("theta_cm_gate", "title", 100, 0., 3.14159);

  hpx = new TH1D("n_px", "title", 100, 0.-2, 0.+2);
  hpy = new TH1D("n_py", "title", 100, 0.-2, 0.+2);
  hpz = new TH1D("n_pz", "title", 100, 0.-2, 0.+2);

  htof = new TH1D("tof", "title", 100, 0.-1, 0.+1);

  return 1;
}

int CAnalyzeNeutGamma::CreateExpData(std::string method, Long64_t nentries)
{
  TTree* tree;
  filesim->GetObject("Koyomi",tree);

  fileexp->cd();
  TTree* newtree = new TTree("Hitagi", "title");

  CDataHimeSim* datasim = 0;
  TBranch *bdatasim = 0;
  if(tree->SetBranchAddress("hime",&datasim,&bdatasim) < 0) return 0;

  CDataHime* dataexp = new CDataHime;
  newtree->Branch("hime", &dataexp);

  topdir->cd();

  Long64_t count = 0;
  if(nentries < 0) nentries = tree->GetEntriesFast();
  for(int i=0; i<nentries; i++) {
    ++count;
    if(count%10000 == 0) std::cout << "\r converted events:" << count << std::flush;

    Long64_t entry = tree->LoadTree(i);
    bdatasim->GetEntry(entry);

    if(!datasim->fEventFlag) continue;

    dataexp->Reset();
    SimToExp(datasim, dataexp, method);

    newtree->Fill();
  }
  std::cout << "\r converted events:" << count << std::endl;

  tree->ResetBranchAddresses();

  return 1;
}

int CAnalyzeNeutGamma::AnalysisExpData(Long64_t nentries)
{
  TTree* treesim;
  filesim->GetObject("Koyomi",treesim);

  CDataHimeSim* datasim = 0;
  TBranch *bdatasim = 0;
  if(treesim->SetBranchAddress("hime",&datasim,&bdatasim) < 0) return 0;

  TTree* treeexp;
  fileexp->GetObject("Hitagi",treeexp);

  CDataHime* dataexp = 0;
  TBranch *bdataexp = 0;
  if(treeexp->SetBranchAddress("hime",&dataexp,&bdataexp) < 0) return 0;

  Long64_t count = 0;
  if(nentries < 0) nentries = treesim->GetEntriesFast();
  for(Long64_t i=0, j=0; i<nentries; i++) {
    Long64_t entrysim = treesim->LoadTree(i);
    bdatasim->GetEntry(entrysim);

    if(!datasim->fEventFlag) continue;

    Long64_t entryexp = treeexp->LoadTree(j++);
    bdataexp->GetEntry(entryexp);

    //protonの真の角度分布をグラフ化してそれと比較する
    Double_t theta_lab, phi_lab, energy_lab;
    Double_t theta_cm, energy_cm;
    if(CAnalyzeTool::GetFirstProtonInformation(datasim, proton) != 0){
      theta_lab = proton->theta();
      phi_lab = proton->phi();
      energy_lab = proton->k();

      neutron->p(datasim->fMomentum.x(),datasim->fMomentum.y(),datasim->fMomentum.z());
      Double_t beta = neutron->p()/(neutron->e()+proton->m());
      CKinema::LorentzBoost(*proton, beta, 2);

      theta_cm = proton->theta();
      energy_cm = proton->k();

      htheta_cm->Fill(theta_cm);
      hthetaenergy_lab->Fill(theta_lab, energy_lab);
      hthetaenergy_cm->Fill(theta_cm, energy_cm);
      if(dataexp->fEventFlag){
	htheta_lab->Fill(theta_lab-dataexp->fPrp.Theta());
	hphi_lab->Fill(phi_lab-dataexp->fPrp.Phi());

	htheta_cm_gate->Fill(theta_cm);
	hthetaenergy_lab_gate->Fill(theta_lab, energy_lab);
	hthetaenergy_cm_gate->Fill(theta_cm, energy_cm);

	hdee_lab->Fill(CAnalyzeTool::GetTotalEnergyLoss(dataexp),
		       energy_lab);
	hthetade_lab->Fill(theta_lab,
			   CAnalyzeTool::GetTotalEnergyLoss(dataexp));
	hthetadee_lab->Fill(theta_lab,
			    CAnalyzeTool::GetTotalEnergyLoss(dataexp),
			    energy_lab);
      }
    }

    if(dataexp->fEventFlag){
      ++count;
      if(count%100 == 0) std::cout << "\r analysed events:" << count << std::flush;
    }else{
      continue;
    }

    //momentum resolution(%)
    hpx->Fill(100*(dataexp->fP.Px()-datasim->fMomentum.Px())/datasim->fMomentum.Mag());
    hpy->Fill(100*(dataexp->fP.Py()-datasim->fMomentum.Py())/datasim->fMomentum.Mag());
    hpz->Fill(100*(dataexp->fP.Pz()-datasim->fMomentum.Pz())/datasim->fMomentum.Mag());

    //ToF resolution(ToF:=(flightlength)*(m0*gamma)/pz)
    neutron->p(dataexp->fP.Px(), dataexp->fP.Py(), dataexp->fP.Pz()); 
    double tofexp = zdepth*neutron->m()*neutron->gamma()/neutron->p(2)/CAnalyzeTool::c;
    neutron->p(datasim->fMomentum.Px(), datasim->fMomentum.Py(), datasim->fMomentum.Pz()); 
    double tofsim = zdepth*neutron->m()*neutron->gamma()/neutron->p(2)/CAnalyzeTool::c;
    htof->Fill(tofexp-tofsim);
  }
  std::cout << "\r analysed events:" << count << std::endl;

  treeexp->ResetBranchAddresses();
  treesim->ResetBranchAddresses();
    
  topdir->cd();

  return 1;
}

int CAnalyzeNeutGamma::DrawHist(Long64_t nentries)
{
  canvas->Divide(3,3);
  canvas->cd(1);
  hpx->Draw();
  hpx->Fit("gaus");
  canvas->cd(2);
  hpy->Draw();
  hpy->Fit("gaus");
  canvas->cd(3);
  hpz->Draw();
  hpz->Fit("gaus");

  canvas->cd(4);
  htheta_lab->Draw();
  htheta_lab->Fit("gaus", "", "same");

  canvas->cd(5);
  hphi_lab->Draw();
  hphi_lab->Fit("gaus", "", "same");

  canvas->cd(6);
  hthetaenergy_lab->Draw("COLZBARBOX");
  canvas->cd(7);
  hthetaenergy_lab_gate->Draw("COLZBARBOX");

  canvas->cd(8);
  htof->Draw();

  TTree* treesim;
  filesim->GetObject("Koyomi",treesim);
  if(nentries < 0) nentries = treesim->GetEntriesFast();

  std::cout << htof->GetEntries() << " " << nentries << std::endl;

  canvas->Update();

  return 1;
}

void CAnalyzeNeutGamma::SimToExp(CDataHimeSim* simdata, CDataHime* dataexp, std::string method)
{
  std::deque<CDataHimeSim::OneData>::iterator itsim;
  itsim = simdata->fDequeOfOneData.begin();
  
  while(itsim != simdata->fDequeOfOneData.end()){
    std::vector<Int_t> id;
    id.push_back(itsim->fIdMother);
    id.push_back(itsim->fId);

    Double_t xl = length/2.+itsim->fPosition.X()-shiftx;
    Double_t xr = length/2.-itsim->fPosition.X()+shiftx;
    Double_t yl = length/2.+itsim->fPosition.Y()-shifty;
    Double_t yr = length/2.-itsim->fPosition.Y()+shifty;

    Double_t fEdepee = CKinema::MeVtoMeVee(itsim->fKineticEnergy,
					   itsim->fKineticEnergy - itsim->fEdep,
					   itsim->fParticleName.Data());

    Double_t tl, tr, al, ar;
    if(rotation == 1){
      if(id[0]%2){
	tl = itsim->fTime + yl/vl;
	tr = itsim->fTime + yr/vl;
	al = fEdepee*exp(-yl/attlength);
	ar = fEdepee*exp(-yr/attlength);
      }else{
	tl = itsim->fTime + xl/vl;
	tr = itsim->fTime + xr/vl;
	al = fEdepee*exp(-xl/attlength);
	ar = fEdepee*exp(-xr/attlength);
      }
    }else{
      tl = itsim->fTime + xl/vl;
      tr = itsim->fTime + xr/vl;
      al = fEdepee*exp(-xl/attlength);
      ar = fEdepee*exp(-xr/attlength);
    }      

    CDataHime::OneData* onedata = dataexp->Find(id[0], id[1]);
    if(onedata){
      if(onedata->fTL > tl) onedata->fTL = tl;
      if(onedata->fTR > tr) onedata->fTR = tr;
      onedata->fAL += al;
      onedata->fAR += ar;
    }else{
      dataexp->fDequeOfOneData.push_back(CDataHime::OneData(id, tl, tr, al, ar));
    }

    ++itsim;
  }

  std::deque<CDataHime::OneData>::iterator it;
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){

    //thr判定その1
    if(thrmethod == 1){
      if(it->fAL < thr || it->fAR < thr){
	it = dataexp->fDequeOfOneData.erase(it);
	continue;
      }
    }

    it->fTL += gRandom->Gaus(0,treso);
    it->fTR += gRandom->Gaus(0,treso);  
    
    neutron->k(ein);
    it->fT = (it->fTL + it->fTR - length/vl)/2/neutron->beta();
    it->fE = sqrt(it->fAL*it->fAR)*exp(length/(2.*attlength));

    //    std::cout << it->fT << " " << it->fAL << " " << it->fAR << " " << it->fE << " " << exp(length/(2.*attlength)) << std::endl;
    
    //thr判定その2
    if(thrmethod == 2){
      if(it->fE < thr){
	it = dataexp->fDequeOfOneData.erase(it);
	continue;
      }
    }

    Double_t x, y, z;
    if(rotation == 1){
      if(it->fId[0]%2){
	x = it->fId[1]*width + width/2. - vlength/2. + shiftx;
	y = vl*(it->fTL-it->fTR)/2. + shifty;
      }else{
	x = vl*(it->fTL-it->fTR)/2. + shiftx;
	y = it->fId[1]*width + width/2. - vlength/2. + shifty;
      }
    }else{
      x = vl*(it->fTL-it->fTR)/2. + shiftx;
      y = it->fId[1]*width + width/2. - vlength/2. + shifty;
    }
    z = it->fId[0]*(depth+gap) + zdepth + depth/2.;

    it->fX.SetXYZ(x, y, z);

    ++it;
  }

  if(method == "Track"){
    Track(dataexp);
  }else if(method == "Simple"){
    Simple(dataexp);
  }else if(method == "Niren"){
    Niren(dataexp);
  }else{
    std::cerr << "method:" << method << " cannot use" << std::endl;
    return;
  }
}

void CAnalyzeNeutGamma::Track(CDataHime* dataexp)
{
#ifdef VISUALIZE
  TGraph2DErrors* graph = new TGraph2DErrors;//staticを付けるとメモリの再確保がなくなり早い
#endif

  if(dataexp->fDequeOfOneData.empty()) return;

  //どうせ何回も使うので
  static std::deque<CDataHime::OneData>::iterator it;
  static std::deque<CDataHime::OneData>::iterator itmin;

  //最速イベントではなく先頭イベント。
  //最速を選ぶと時間分解能の分だけ微妙にシフトすると思われる。
  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    //    if(it->fX.Z() < itmin->fX.Z()) itmin = it;
    if(it->fX.Z() == itmin->fX.Z()){
      if(it->fT < itmin->fT) itmin = it;
    }else if(it->fX.Z() < itmin->fX.Z()){
      itmin = it;
    }
    ++it;
  }

  //遅いイベントは関係ないので切り捨て。
  it = dataexp->fDequeOfOneData.begin();
  double tmin = itmin->fT;//roop内部でeraseを使ってるので途中でitminが不正になりうる。
  while(it != dataexp->fDequeOfOneData.end()){
    //先頭でhitしてからある程度時間がかかったeventは切ってしまう
    //時間で排除する代わりに、シグナルの近傍度みたいな量を定義して、
    //それでcutしたほうがノイズを効率的に落とせるかも
    if(abs(it->fT - tmin) > 1.2){
      it = dataexp->fDequeOfOneData.erase(it);
      continue;
    }
    ++it;
  }

  //eraseしたので先頭イベントのイテレータを取り直す。(必須!!)
  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    if(it->fX.Z() < itmin->fX.Z()) itmin = it;
    //    if(it->fT < itmin->fT) itmin = it;
    ++it;
  }

  //hit数に満たない場合はcut
  if((int)dataexp->fDequeOfOneData.size() < multihit){
    dataexp->fEventFlag = 0;//in-validなデータの場合は0
    return;
  }

  //2Dfit(zの誤差を無視する)でtracking
  Int_t point = 0;
  std::vector<double> pointx, pointy, pointz;
  std::vector<double> pointxe, pointye, pointze;
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    pointx.push_back(it->fX.X());
    pointy.push_back(it->fX.Y());
    pointz.push_back(it->fX.Z());
    if(it->fId[0]%2){
      pointxe.push_back(width/sqrt(12));
      pointye.push_back(vl*treso/sqrt(2));
    }else{	 
      pointxe.push_back(vl*treso/sqrt(2));
      pointye.push_back(width/sqrt(12));
    }
    pointze.push_back(depth/sqrt(12));

#ifdef VISUALIZE
    graph->SetPoint(point, it->fX.X(), it->fX.Y(), it->fX.Z());
    if(it->fId[0]%2){
      graph->SetPointError(point, width/sqrt(12), vl*treso/sqrt(2), depth/sqrt(12));
    }else{	 
      graph->SetPointError(point, vl*treso/sqrt(2), width/sqrt(12), depth/sqrt(12));
    }
#endif

    ++point;
    ++it;
  }
  double parFit[4];

  CAnalysis::LinearFit(pointz, pointx, pointxe, parFit[0], parFit[1]);
  CAnalysis::LinearFit(pointz, pointy, pointye, parFit[2], parFit[3]);
  double aminx = CAnalysis::SD(pointz, pointx, parFit[0], parFit[1]);
  double aminy = CAnalysis::SD(pointz, pointy, parFit[2], parFit[3]);
  double amin = sqrt(aminx*aminx+aminy*aminy);//rmsっぽいもの

  //rmsっぽいものを収集
  hamin->Fill(amin);

  //理想的には選択的にトラックを引くアルゴリズムが必要。
  //たとえばamin<5になるまで、計算に入れる点を変えるとか。
  //今はaminminで単純に切り捨てごめん。
  if(amin > aminmin){
    dataexp->fEventFlag = 0;//in-validなデータの場合は0
    return;
  }

  //analysis of timing
  TVector3 xf(parFit[1], parFit[3], 1.);//fitした直線にそったベクトル
  double theta = xf.Angle(itmin->fX);
  double theta_rp = xf.Theta();
  double phi_rp = xf.Phi();

  // derive recoile proton energy from theta(lab)
  CParticle pin = CParticleDefinition::GetParticle("proton");
  CParticle nin = CParticleDefinition::GetParticle("neutron");
  CParticle pout = CParticleDefinition::GetParticle("proton");
  CParticle nout = CParticleDefinition::GetParticle("neutron");

  //cal ein by simple method
  TVector3 beta = 1./(itmin->fT*CAnalyzeTool::c)*itmin->fX;
  TVector3 momentum = nin.m()*1./sqrt(1.-beta.Mag()*beta.Mag())*beta;

  nin.p(momentum.Px(), momentum.Py(), momentum.Pz());
  nin.k(nin.k()-sp);
  //  nin.k(ein-sp);//一陽子分離エネルギーを考慮。ほとんど効かないけど。
  pin.k(0);//Fermi motionは無視して静止していると考える。

  //lab系のthetaをパラメータとして、剛体の2体散乱をさせる。
  Double_t theta_cm = CKinema::Scatter(theta, 0., nin, pin, pout, nout);

  //pout,noutの角度を修正(上記のthetaはlab系でのneutronとの角度の差)
  pout.thetaphi(theta_rp, phi_rp);
  nout.thetaphibar(theta_rp, phi_rp);

  //recoil protonの情報の保存
  dataexp->fPrp.SetPxPyPzE(pout.p(0), pout.p(1), pout.p(2), pout.e());

  //thetaを取得
  htheta_exp->Fill(theta);
  htheta_cm_exp->Fill(theta_cm);

  //timingの解析の本体
  double timing = 0;
  int count = 0;
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    double tsum = -1.;
    if(it == itmin){
      tsum = 0;
      //先頭でhitしてから4層通過分程度時間がかかったeventは時間の計算には参加させない
    }else if(abs(it->fT - itmin->fT) < 0.8){
      double xtemp, ytemp, ztemp;
      if(it->fId[0]-itmin->fId[0] > 0){
	//二層め以降で成ったシグナルは荷電粒子によるものと思われ、
	//シミュレーション的にはシンチレータの端面で鳴るのでZを補正。
	//実験でも基本的に同じ補正でいいはずだが未知
	CAnalysis::line(it->fX.Z()-depth/2., parFit, xtemp, ytemp, ztemp);
      }else{
	CAnalysis::line(it->fX.Z(), parFit, xtemp, ytemp, ztemp);	
      }
      TVector3 X(xtemp, ytemp, ztemp);
      CAnalysis::line(itmin->fX.Z(), parFit, xtemp, ytemp, ztemp);
      TVector3 Xmin(xtemp, ytemp, ztemp);
      TVector3 distance_v = X - Xmin;
      double distance = distance_v.Mag();

      CParticle ptemp = pout;

      //検出器中を飛んだ時間の計算。先に物質中を飛んだ時間をだして、
      //あとから残りの空間を飛んだ時間を足す。
      //かなり荒いけど結構まともな近似
      //      tsum = CKinema::BetheBloch::TimeOut
      //	("p", "Scint", ptemp,
      //	 distance - (it->fId[0]-itmin->fId[0])*gap/cos(distance_v.Theta()), step);
      //      if(tsum >= 0.){
      //	//ほんとは2cm+2cm+2cm+....と交互に時間を足していかなければならない。
      //	//ここで少しmeanのずれが生じていると考えられる。
      //	tsum += (distance/2.)/((pout.beta()+ptemp.beta())*CKinema::fLightVelocity/2.);
      //      }

      //こっちの計算は真面目に計算した場合。エネルギーでのカットオフが必要
      double flightcos = cos(distance_v.Theta());
      int flightcount = 0;
      double flightdistance;

      tsum = 0;//初期値はtsum=-1なので0にしておく

      while(1){
	if((flightcount % 2) == 0){
	  if(flightcount == 0){
	    flightdistance = depth/2/flightcos;//一枚目は真ん中で発生したと考える
	  }else{
	    flightdistance = depth/flightcos;
	  }
	}else{
	  flightdistance = gap/flightcos;	    
	}

	if(distance < flightdistance) flightdistance = distance;

	if((flightcount % 2) == 0){
	  double tsumtemp;
	  tsumtemp = CKinema::BetheBloch::TimeOut("p", "Scint", ptemp, flightdistance, step);
	  //止まってしまった場合
	  if(tsumtemp < 0){
	    tsum = -1;
	    break;
	  }
	  tsum += tsumtemp;
	}else{
	  tsum += flightdistance/(ptemp.beta()*CKinema::fLightVelocity);
	}

	//ひょろひょろだったらその点を使った計算は不確定なのでいらない
	if(ptemp.k() < 20){
	  tsum = -1;
	  break;
	}

	distance -= flightdistance;
	if(distance <= 0) break;//発光点まで来たので終了

	++flightcount;
      }
    }
    if(tsum >= 0){
      //補正した結果、改善する方向に向かわない場合は補正に参加させない
      if(abs((it->fT - tsum) - itmin->fT) < treso*2){
	timing += it->fT - tsum;
	++count;
      }
    }
    ++it;
  }
  timing /= count;

  //位置はlinearfitで求まった先頭、時間は上の解析で補正した時間
  double xtemp, ytemp, ztemp;
  CAnalysis::line(itmin->fX.Z(),parFit,xtemp,ytemp,ztemp);
  dataexp->fX.SetXYZT(xtemp, ytemp, ztemp, timing);

  //X,Y,Z,Tが求まったらMomentumを計算しておく。
  //この関数の内部で非数が渡されてないかエラーチェックが走る
  dataexp->CalMomentum();

#ifdef VISUALIZE
  graph->SetMarkerStyle(8);
  graph->SetMarkerSize(1);
  graph->GetXaxis()->SetLimits(-length/2., length/2.);
  graph->GetYaxis()->SetLimits(-length/2., length/2.);
  graph->SetMinimum(zdepth);
  graph->SetMaximum(zdepth+zdepth_back);

  if(0){
    graph->Draw("PERR");
  }else{
    static bool init = true;
    if(init){
      graph->Draw("PERR");
      init = false;
    }else{
      graph->Draw("PERRSAME");
    }
  }

  CAnalysis::DrawLine(parFit, kRed, zdepth, zdepth_back);

  canvas = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("canvas");
  canvas->Update();

  CAnalysis::EventLoop();
#endif

  dataexp->fEventFlag = 1;//validなデータの場合は1

  return;
}

void CAnalyzeNeutGamma::Simple(CDataHime* dataexp)
{
  static std::deque<CDataHime::OneData>::iterator it;
  static std::deque<CDataHime::OneData>::iterator itmin;

  if(dataexp->fDequeOfOneData.empty()){
    dataexp->fEventFlag = 0;
    return;
  }

  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  dataexp->fX.SetXYZT(itmin->fX.X(), itmin->fX.Y(), itmin->fX.Z(), itmin->fT);

  dataexp->CalMomentum();

  dataexp->fEventFlag = 1;

  return;
}

void CAnalyzeNeutGamma::Niren(CDataHime* dataexp)
{
  static std::deque<CDataHime::OneData>::iterator it;
  static std::deque<CDataHime::OneData>::iterator itmin;

  if(dataexp->fDequeOfOneData.empty()){
    dataexp->fEventFlag = 0;
    return;
  }

  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  //ここのアルゴリズムがかなり適当なのでだめだめ。
  //本気で2連hitを使うならdtから得たyとの整合性を取ったりしないとだめすぎる。
  Double_t zmin = itmin->fX.Z() - zdepth+1;
  Int_t idmin = zmin/2./depth+0.1;
  std::deque<CDataHime::OneData>::iterator itmin2;
  itmin2 = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    Double_t z = it->fX.Z() - (zdepth+1);
    Int_t id = z/2./depth+0.1;
    if(idmin+1 == id){
      itmin2 = it;
      break;
    }
    ++it;
  }
  
  if(itmin2 == dataexp->fDequeOfOneData.begin()){
    dataexp->fEventFlag = 0;
    return;
  }
  
  if(idmin%2){
    dataexp->fX.SetX(itmin->fX.X());
    dataexp->fX.SetY(itmin2->fX.Y());
  }else{
    dataexp->fX.SetX(itmin2->fX.X());
    dataexp->fX.SetY(itmin->fX.Y());
  }
  dataexp->fX.SetZ(itmin->fX.Z());

// ノイズが多すぎてうまく行かない
//  Double_t meanbeta = 0.6;
//  Double_t meanlength = 4.;
//  Double_t time;
//  if(itmin->fX.Z() - itmin2->fX.Z() < 1.){
//    time = (itmin->fT + itmin2->fT)/2.;
//  }else{    
//    time = (itmin->fT + itmin2->fT)/2. - meanlength/(meanbeta*CKinema::fLightVelocity);
//  }
//  dataexp->fX.SetT(time);

  //シンプルに先頭の時間
  dataexp->fX.SetT(itmin->fT);

  dataexp->CalMomentum();

  dataexp->fEventFlag = 1;

  return;
}
