
#include "TROOT.h"
#include "TStyle.h"
#include "TFile.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TH3D.h"
#include "TGraph2DErrors.h"
#include "TPolyLine3D.h"
#include "TVector3.h"
#include "TF1.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"

#include <iostream>
#include <string>
#include <cmath>
#include <deque>
#include <fstream>
#include <algorithm>

#include "CDataHime.hh"
#include "CDataHimeSim.hh"
#include "CKinema.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"
#include "FFunction.hh"
#include "CMapFunction1D.hh"
#include "CFold.hh"
#include "CAnalysis.hh"
#include "CXmlManager.hh"

#include "CAnalyzeTool.hh"
#include "CAnalyzeNeutRCNP2n.hh"

ClassImp(CAnalyzeNeutRCNP2n)

//#define VISUALIZE 1

CAnalyzeNeutRCNP2n::CAnalyzeNeutRCNP2n(std::string rootfile)
{
  canvas = new TCanvas("canvas","",200,10,900,900);
  canvas->SetFillColor(0);

  topdir = gDirectory;

  filesim = TFile::Open(rootfile.c_str(), "READ");
  if(!filesim) abort();

  fileexp = TFile::Open("root/exp.root","recreate");
  if(!fileexp) abort();

  proton = new CParticle(CParticleDefinition::GetParticle("proton"));
  neutron = new CParticle(CParticleDefinition::GetParticle("neutron"));
}

CAnalyzeNeutRCNP2n::~CAnalyzeNeutRCNP2n()
{
  delete neutron;
  delete proton;

  fileexp->Write();
  fileexp->Close();
  filesim->Close();
}

void CAnalyzeNeutRCNP2n::Main(std::string prmfile, std::string detector, std::string method, Long64_t nentries)
{
  if(!ReadParamaters(prmfile, detector)){
    std::cerr << "error occur during reading paramaters" << std::endl;
    return;
  }

  if(!CreateHist()){
    std::cerr << "error occur during creating hist" << std::endl;
    return;
  }    

  if(!CreateExpData(method, nentries)){
    std::cerr << "error occur during creating experiment data" << std::endl;
    return;
  }    

  if(!AnalysisExpData(nentries)){
    std::cerr << "error occur during analyzing experiment data" << std::endl;
    return;
  }    

  if(!DrawHist(nentries)){
    std::cerr << "error occur during drawing histogram" << std::endl;
    return;
  }    
}

int CAnalyzeNeutRCNP2n::ReadParamaters(std::string prmfile, std::string detector)
{
  CXmlManager xml;
  if (!xml.OpenFile(prmfile)) return 0;

  if (!xml.GetValue("//"+detector+"//thrmethod", &thrmethod)) return 0;
  if (!xml.GetValue("//"+detector+"//thr", &thr)) return 0;
  if (!xml.GetValue("//"+detector+"//treso", &treso)) return 0;
  if (!xml.GetValue("//"+detector+"//attlength", &attlength)) return 0;
  if (!xml.GetValue("//"+detector+"//vl", &vl)) return 0;
  if (!xml.GetValue("//"+detector+"//rotation", &rotation)) return 0;
  if (!xml.GetValue("//"+detector+"//length", &length)) return 0;
  if (!xml.GetValue("//"+detector+"//vlength", &vlength)) return 0;
  if (!xml.GetValue("//"+detector+"//width", &width)) return 0;
  if (!xml.GetValue("//"+detector+"//depth", &depth)) return 0;
  if (!xml.GetValue("//"+detector+"//gap", &gap)) return 0;
  if (!xml.GetValue("//"+detector+"//zdepth", &zdepth)) return 0;
  if (!xml.GetValue("//"+detector+"//zdepth_back", &zdepth_back)) return 0;
  if (!xml.GetValue("//"+detector+"//shiftx", &shiftx)) return 0;
  if (!xml.GetValue("//"+detector+"//shifty", &shifty)) return 0;
  if (!xml.GetValue("//"+detector+"//multihit", &multihit)) return 0;
  if (!xml.GetValue("//"+detector+"//aminmin", &aminmin)) return 0;
  if (!xml.GetValue("//"+detector+"//ein", &ein)) return 0;
  if (!xml.GetValue("//"+detector+"//sp", &sp)) return 0;
  if (!xml.GetValue("//"+detector+"//step", &step)) return 0;

  return 1;
}

int CAnalyzeNeutRCNP2n::CreateHist()
{
  topdir->cd();

  hpx = new TH1D("n_px", "title", 100, 0.-2, 0.+2);
  hpy = new TH1D("n_py", "title", 100, 0.-2, 0.+2);
  hpz = new TH1D("n_pz", "title", 100, 0.-2, 0.+2);
  hp2x = new TH2D("n_p2x", "title", 100, 0.-2, 0.+2, 100, 0.-2, 0.+2);
  hp2y = new TH2D("n_p2y", "title", 100, 0.-2, 0.+2, 100, 0.-2, 0.+2);
  hp2z = new TH2D("n_p2z", "title", 100, 0.-2, 0.+2, 100, 0.-2, 0.+2);
  hp2xgate = new TH2D("n_p2xgate", "title", 100, 0.-2, 0.+2, 100, 0.-2, 0.+2);
  hp2ygate = new TH2D("n_p2ygate", "title", 100, 0.-2, 0.+2, 100, 0.-2, 0.+2);
  hp2zgate = new TH2D("n_p2zgate", "title", 100, 0.-2, 0.+2, 100, 0.-2, 0.+2);

  return 1;
}

int CAnalyzeNeutRCNP2n::CreateExpData(std::string method, Long64_t nentries)
{
  TTree* tree;
  filesim->GetObject("Koyomi",tree);

  fileexp->cd();
  TTree* newtree = new TTree("Hitagi", "title");

  CDataHimeSim* datasim = 0;
  TBranch *bdatasim = 0;
  if(tree->SetBranchAddress("hime",&datasim,&bdatasim) < 0) return 0;

  CDataHime* dataexp = new CDataHime;
  newtree->Branch("hime", &dataexp);

  topdir->cd();

  Long64_t count = 0;
  if(nentries < 0) nentries = tree->GetEntriesFast();
  for(int i=0; i<nentries; i++) {
    ++count;
    if(count%1000 == 0) std::cout << "\r converted events:" << count << std::flush;

    Long64_t entry = tree->LoadTree(i);
    bdatasim->GetEntry(entry);

    if(!datasim->fEventFlag) continue;

    dataexp->Reset();
    SimToExp(datasim, dataexp, method);

    newtree->Fill();
  }
  std::cout << "\r converted events:" << count << std::endl;

  tree->ResetBranchAddresses();

  return 1;
}

int CAnalyzeNeutRCNP2n::AnalysisExpData(Long64_t nentries)
{
  TTree* treesim;
  filesim->GetObject("Koyomi",treesim);

  CDataHimeSim* datasim = 0;
  TBranch *bdatasim = 0;
  if(treesim->SetBranchAddress("hime",&datasim,&bdatasim) < 0) return 0;

  TTree* treeexp;
  fileexp->GetObject("Hitagi",treeexp);

  CDataHime* dataexp = 0;
  TBranch *bdataexp = 0;
  if(treeexp->SetBranchAddress("hime",&dataexp,&bdataexp) < 0) return 0;

  Long64_t count = 0;
  if(nentries < 0) nentries = treesim->GetEntriesFast();
  for(Long64_t i=0, j=0; i<nentries; i++) {
    Long64_t entrysim = treesim->LoadTree(i);
    bdatasim->GetEntry(entrysim);

    if(!datasim->fEventFlag) continue;

    Long64_t entryexp = treeexp->LoadTree(j++);
    bdataexp->GetEntry(entryexp);

    if(dataexp->fEventFlag){
      ++count;
      if(count%100 == 0) std::cout << "\r analysed events:" << count << std::flush;
    }else{
      continue;
    }

    //momentum resolution(%)
    double dp11x = 100*(dataexp->fP.Px()-datasim->fMomentum.Px())/datasim->fMomentum.Mag();
    double dp11y = 100*(dataexp->fP.Py()-datasim->fMomentum.Py())/datasim->fMomentum.Mag();
    double dp11z = 100*(dataexp->fP.Pz()-datasim->fMomentum.Pz())/datasim->fMomentum.Mag();
    double dp22x = 100*(dataexp->fP2.Px()-datasim->fMomentum2.Px())/datasim->fMomentum2.Mag();
    double dp22y = 100*(dataexp->fP2.Py()-datasim->fMomentum2.Py())/datasim->fMomentum2.Mag();
    double dp22z = 100*(dataexp->fP2.Pz()-datasim->fMomentum2.Pz())/datasim->fMomentum2.Mag();
    double dp12x = 100*(dataexp->fP.Px()-datasim->fMomentum2.Px())/datasim->fMomentum2.Mag();
    double dp12y = 100*(dataexp->fP.Py()-datasim->fMomentum2.Py())/datasim->fMomentum2.Mag();
    double dp12z = 100*(dataexp->fP.Pz()-datasim->fMomentum2.Pz())/datasim->fMomentum2.Mag();
    double dp21x = 100*(dataexp->fP2.Px()-datasim->fMomentum.Px())/datasim->fMomentum.Mag();
    double dp21y = 100*(dataexp->fP2.Py()-datasim->fMomentum.Py())/datasim->fMomentum.Mag();
    double dp21z = 100*(dataexp->fP2.Pz()-datasim->fMomentum.Pz())/datasim->fMomentum.Mag();

    double dp11 = dp11x*dp11x+dp11y*dp11y+dp11z*dp11z+dp22x*dp22x+dp22y*dp22y+dp22z*dp22z;
    double dp22 = dp12x*dp12x+dp12y*dp12y+dp12z*dp12z+dp21x*dp21x+dp21y*dp21y+dp21z*dp21z;

//    std::cout << "start "; 
//    std::cout << " " <<dataexp->fP.Px();
//    std::cout << " " <<dataexp->fP.Py();
//    std::cout << " " <<dataexp->fP.Pz();
//    std::cout << " " <<dataexp->fP2.Px();
//    std::cout << " " <<dataexp->fP2.Py();
//    std::cout << " " <<dataexp->fP2.Pz();
//    std::cout << " " <<datasim->fMomentum.Px();
//    std::cout << " " <<datasim->fMomentum.Py();
//    std::cout << " " <<datasim->fMomentum.Pz();
//    std::cout << " " <<datasim->fMomentum2.Px();
//    std::cout << " " <<datasim->fMomentum2.Py();
//    std::cout << " " <<datasim->fMomentum2.Pz();
//    std::cout << std::endl;

//    if(0){
    if(1){
      if(dataexp->fP2.Px() != 0 &&
	 dataexp->fP2.Py() != 0 &&
	 dataexp->fP2.Pz() != 0){
	hp2x->Fill(dp11x, dp11x);
	hp2y->Fill(dp11y, dp11y);
	hp2z->Fill(dp11z, dp11z);
      }else{
	hpx->Fill(dp11x);
	hpy->Fill(dp11y);
	hpz->Fill(dp11z);
      }
    }else{
      if(dp11 < dp22){
	if(dataexp->fP2.Px() != 0 &&
	   dataexp->fP2.Py() != 0 &&
	   dataexp->fP2.Pz() != 0){
	  hp2x->Fill(dp11x, dp22x);
	  hp2y->Fill(dp11y, dp22y);
	  hp2z->Fill(dp11z, dp22z);
	  if(abs(dp11x) < 1 && abs(dp22x) < 1 &&
	     abs(dp11y) < 1 && abs(dp22y) < 1 &&
	     abs(dp11z) < 2 && abs(dp22z) < 2){
	    hp2xgate->Fill(dp11x, dp22x);
	    hp2ygate->Fill(dp11y, dp22y);
	    hp2zgate->Fill(dp11z, dp22z);
	  }
	}else{
	  hpx->Fill(dp11x);
	  hpy->Fill(dp11y);
	  hpz->Fill(dp11z);
	}
      }else{
	if(dataexp->fP2.Px() != 0 &&
	   dataexp->fP2.Py() != 0 &&
	   dataexp->fP2.Pz() != 0){
	  hp2x->Fill(dp12x, dp21x);
	  hp2y->Fill(dp12y, dp21y);
	  hp2z->Fill(dp12z, dp21z);
	  if(abs(dp12x) < 1 && abs(dp21x) < 1 &&
	     abs(dp12y) < 1 && abs(dp21y) < 1 &&
	     abs(dp12z) < 2 && abs(dp21z) < 2){
	    hp2xgate->Fill(dp12x, dp21x);
	    hp2ygate->Fill(dp12y, dp21y);
	    hp2zgate->Fill(dp12z, dp21z);
	  }
	}else{
	  hpx->Fill(dp12x);
	  hpy->Fill(dp12y);
	  hpz->Fill(dp12z);
	}
      }
    }
  }
  std::cout << "\r analysed events:" << count << std::endl;

  treeexp->ResetBranchAddresses();
  treesim->ResetBranchAddresses();
    
  topdir->cd();

  return 1;
}

int CAnalyzeNeutRCNP2n::DrawHist(Long64_t /*nentries*/)
{
  canvas->Divide(3,3);
  canvas->cd(1);
  hpx->Draw();
  hpx->Fit("gaus");
  canvas->cd(2);
  hpy->Draw();
  hpy->Fit("gaus");
  canvas->cd(3);
  hpz->Draw();
  hpz->Fit("gaus");

  canvas->cd(4);
  hp2x->Draw();
  canvas->cd(5);
  hp2y->Draw();
  canvas->cd(6);
  hp2z->Draw();

  canvas->cd(7);
  hp2xgate->Draw();
  canvas->cd(8);
  hp2ygate->Draw();
  canvas->cd(9);
  hp2zgate->Draw();

  canvas->Update();

  return 1;
}

void CAnalyzeNeutRCNP2n::SimToExp(CDataHimeSim* simdata, CDataHime* dataexp, std::string method)
{
  std::deque<CDataHimeSim::OneData>::iterator itsim;
  itsim = simdata->fDequeOfOneData.begin();
  while(itsim != simdata->fDequeOfOneData.end()){
    std::vector<Int_t> id;
    id.push_back(itsim->fIdMother);
    id.push_back(itsim->fId);

    Double_t xl = length/2.+itsim->fPosition.X()-shiftx;
    Double_t xr = length/2.-itsim->fPosition.X()+shiftx;
    Double_t yl = length/2.+itsim->fPosition.Y()-shifty;
    Double_t yr = length/2.-itsim->fPosition.Y()+shifty;

    Double_t fEdepee = CKinema::MeVtoMeVee(itsim->fKineticEnergy,
					   itsim->fKineticEnergy - itsim->fEdep,
					   itsim->fParticleName.Data());

    Double_t tl, tr, al, ar;
    if(rotation == 1){
      if(id[0]%2){
	tl = itsim->fTime + yl/vl;
	tr = itsim->fTime + yr/vl;
	al = fEdepee*exp(-yl/attlength);
	ar = fEdepee*exp(-yr/attlength);
      }else{
	tl = itsim->fTime + xl/vl;
	tr = itsim->fTime + xr/vl;
	al = fEdepee*exp(-xl/attlength);
	ar = fEdepee*exp(-xr/attlength);
      }
    }else{
      tl = itsim->fTime + xl/vl;
      tr = itsim->fTime + xr/vl;
      al = fEdepee*exp(-xl/attlength);
      ar = fEdepee*exp(-xr/attlength);
    }      

    CDataHime::OneData* onedata = dataexp->Find(id[0], id[1]);
    if(onedata){
      if(onedata->fTL > tl) onedata->fTL = tl;
      if(onedata->fTR > tr) onedata->fTR = tr;
      onedata->fAL += al;
      onedata->fAR += ar;
    }else{
      dataexp->fDequeOfOneData.push_back(CDataHime::OneData(id, tl, tr, al, ar));
    }

    ++itsim;
  }

  std::deque<CDataHime::OneData>::iterator it;
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){

    //thr判定その1
    if(thrmethod == 1){
      if(it->fAL < thr || it->fAR < thr){
	it = dataexp->fDequeOfOneData.erase(it);
	continue;
      }
    }

    it->fTL += gRandom->Gaus(0,treso);
    it->fTR += gRandom->Gaus(0,treso);  
    
    it->fT = (it->fTL + it->fTR - length/vl)/2;
    it->fE = sqrt(it->fAL*it->fAR)*exp(length/(2.*attlength));
    
    //thr判定その2
    if(thrmethod == 2){
      if(it->fE < thr){
	it = dataexp->fDequeOfOneData.erase(it);
	continue;
      }
    }

    Double_t x, y, z;
    if(rotation == 1){
      if(it->fId[0]%2){
	x = it->fId[1]*width + width/2. - vlength/2. + shiftx;
	y = vl*(it->fTL-it->fTR)/2. + shifty;
      }else{
	x = vl*(it->fTL-it->fTR)/2. + shiftx;
	y = it->fId[1]*width + width/2. - vlength/2. + shifty;
      }
    }else{
      x = vl*(it->fTL-it->fTR)/2. + shiftx;
      y = it->fId[1]*width + width/2. - vlength/2. + shifty;
    }
    z = it->fId[0]*(depth+gap) + zdepth + depth/2.;

    it->fX.SetXYZ(x, y, z);

    ++it;
  }

  //2n探索_____________________________________________

  bool (CAnalyzeNeutRCNP2n::*analysismethod)(std::deque<CDataHime::OneData> *data, double &x, double &y, double &z, double &t);
  if(method == "Track"){
    analysismethod = &CAnalyzeNeutRCNP2n::Track;
  }else if(method == "Simple"){
    analysismethod = &CAnalyzeNeutRCNP2n::Simple;
  }else{
    std::cerr << "method:" << method << " cannot use" << std::endl;
    return;
  }

  if(dataexp->fDequeOfOneData.empty()) return;

  //  std::cout << "\nstart" << std::endl;

  //孤立点の除去
  EliminateSolitaryPoint(&dataexp->fDequeOfOneData, 1.8);

  //1Dfitに満たない場合はcut
  if((int)dataexp->fDequeOfOneData.size() < multihit){
    dataexp->fEventFlag = 0;//in-validなデータの場合は0
    return;
  }

  //group化
  std::deque<std::deque<CDataHime::OneData> > data_array;
  Grouping(&dataexp->fDequeOfOneData, &data_array);
  //  std::cout << data_array.size() << std::endl;

  //groupごとに孤立点を除去
  std::deque<std::deque<CDataHime::OneData> >::iterator it_data_array = data_array.begin();
  while(it_data_array != data_array.end()){
    std::deque<CDataHime::OneData> *data = &(*it_data_array);
    //    EliminateSolitaryPoint(data, 1.8); // group別けしたときに孤立しているはず
    if((int)data->size() < multihit){
      it_data_array = data_array.erase(it_data_array);
//    }else if((int)data->size() == 3){
//      double parFit[4];
//      double amin = OneTrackFit(data, parFit);
//      //      std::cout << amin << std::endl;
//      if(amin > 5){//これぐらいずれてるときは3点が直線になっていなくて二層のみで3点を作っている。
//	it_data_array = data_array.erase(it_data_array);
//      }else{
//	++it_data_array;
//      }
    }else{
      ++it_data_array;
    }
  }

  dataexp->ClearMomentum();

#ifdef VISUALIZE
  static TGraph2DErrors* graph = 0;
  delete graph;
  graph = new TGraph2DErrors;//staticを付けるとメモリの再確保がなくなり早い
  Int_t point = 0;
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    graph->SetPoint(point, it->fX.X(), it->fX.Y(), it->fX.Z());
    if(it->fId[0]%2){
      graph->SetPointError(point, width/sqrt(12), vl*treso/sqrt(2), depth/sqrt(12));
    }else{	 
      graph->SetPointError(point, vl*treso/sqrt(2), width/sqrt(12), depth/sqrt(12));
    }
    ++point;
    ++it;
  }

  graph->SetMarkerStyle(8);
  graph->SetMarkerSize(1);
  graph->GetXaxis()->SetLimits(-vlength/1.5, vlength/1.5);
  graph->GetYaxis()->SetLimits(-vlength/1.5, vlength/1.5);
  graph->SetMinimum(zdepth);
  graph->SetMaximum(zdepth+zdepth_back);

  //  if(0){
  if(1){
    graph->Draw("PERR");
  }else{
    static bool init = true;
    if(init){
      graph->Draw("PERR");
      init = false;
    }else{
      graph->Draw("PERRSAME");
    }
  }
#endif

  bool crosstalk = false;
  if(data_array.size() == 1){//1group
    std::deque<CDataHime::OneData> data1;
    data1 = data_array[0];

    //1nと思ってfit
    double parFit[4];
    double amin1 = OneTrackFit(&data1, parFit);
//    if(amin1 > aminmin){//これぐらいずれてるときは大体おかしい?
//      dataexp->fEventFlag = 0;
//      return;
//    }

    double x1, y1, z1, t1;
    if(!(this->*analysismethod)(&data1, x1, y1, z1, t1)){
      dataexp->fEventFlag = 0;
      return;
    }
    dataexp->fX.SetXYZT(x1, y1, z1, t1);
    dataexp->CalMomentum();

#ifdef VISUALIZE
    CDataHime::OneData onedata;
    onedata = FrontData(&data1);
    double z1f = onedata.fX.Z();
    onedata = BackData(&data1);
    double z1b = onedata.fX.Z();

    CAnalysis::DrawLine(parFit, kRed,  z1f-2, z1b-z1f+4);
#endif

    dataexp->fEventFlag = 1;
  }else if(data_array.size() == 2){ //2group
    std::deque<CDataHime::OneData> data1, data2;
    data1 = data_array[0];
    data2 = data_array[1];

    //最小だったaminに対してfit結果をとってくる(面倒なので再度fit)
    //TwoTrackFitでも同じ結果を出せるがシンプルに
    double parFit1[4], parFit2[4];
    double amin2_1 = OneTrackFit(&data1, parFit1);
    double amin2_2 = OneTrackFit(&data2, parFit2);
    double amin2 = sqrt(amin2_1*amin2_1+amin2_2*amin2_2);

    // cross talk の除去
    crosstalk = true;
    if(CrossTalk(&data1, &data2)){
      crosstalk = true;
      dataexp->fEventFlag = 0;
      return;
    }
//    if(amin2 > aminmin){//これぐらいずれてるときは大体おかしい?
//      dataexp->fEventFlag = 0;
//      return;
//    }
    double x1, y1, z1, t1;
    double x2, y2, z2, t2;
    if(!(this->*analysismethod)(&data1, x1, y1, z1, t1)){
      std::cout << "aaa" << std::endl;
      dataexp->fEventFlag = 0;
      return;
    }
    if(!(this->*analysismethod)(&data2, x2, y2, z2, t2)){
      std::cout << "aaa" << std::endl;
      dataexp->fEventFlag = 0;
      return;
    }

    dataexp->fX.SetXYZT(x1, y1, z1, t1);
    dataexp->fX2.SetXYZT(x2, y2, z2, t2);

    //X,Y,Z,Tが求まったらMomentumを計算しておく。
    //この関数の内部で非数が渡されてないかエラーチェックが走る
    dataexp->CalMomentum();
    dataexp->CalMomentum2();

#ifdef VISUALIZE
    CDataHime::OneData onedata;
    onedata = FrontData(&data1);
    double z1f = onedata.fX.Z();
    onedata = BackData(&data1);
    double z1b = onedata.fX.Z();
    onedata = FrontData(&data2);
    double z2f = onedata.fX.Z();
    onedata = BackData(&data2);
    double z2b = onedata.fX.Z();
    CAnalysis::DrawLine(parFit1, kRed,  z1f-2, z1b-z1f+4);
    CAnalysis::DrawLine(parFit2, kBlue, z2f-2, z2b-z2f+4);
#endif

    dataexp->fEventFlag = 1;
  }else if(data_array.size() == 3){ //3group
    static int i = 0;
    //    std::cout << "3hit " << ++i << std::endl;
    dataexp->fEventFlag = 0;
    return;
  }else{
    static int i = 0;
    //    std::cout << "4hit " << ++i << " " << data_array.size() << " " << std::endl;
    dataexp->fEventFlag = 0;
    return;
  }

#ifdef VISUALIZE
  if(crosstalk){
    canvas->Update();
    CAnalysis::EventLoop();
  }
#endif

  return;
}

bool CAnalyzeNeutRCNP2n::Track(std::deque<CDataHime::OneData> *data, double &x, double &y, double &z, double &t)
{
  if(data->empty()) return false;

  //どうせ何回も使うので
  static std::deque<CDataHime::OneData>::iterator it;
  static std::deque<CDataHime::OneData>::iterator itmin;

  //最速イベントではなく先頭イベント。
  //最速を選ぶと時間分解能の分だけ微妙にシフトすると思われる。
  itmin = data->begin();
  it = data->begin();
  while(it != data->end()){
    //    if(it->fX.Z() < itmin->fX.Z()) itmin = it;
    if(it->fX.Z() == itmin->fX.Z()){
      if(it->fT < itmin->fT) itmin = it;
    }else if(it->fX.Z() < itmin->fX.Z()){
      itmin = it;
    }
    ++it;
  }

// Simpleとイベント数が変わると面倒なので切り捨てはしない
//
//  //遅いイベントは関係ないので切り捨て。
//  it = data->begin();
//  double tmin = itmin->fT;//roop内部でeraseを使ってるので途中でitminが不正になりうる。
//  while(it != data->end()){
//    //先頭でhitしてからある程度時間がかかったeventは切ってしまう
//    //時間で排除する代わりに、シグナルの近傍度みたいな量を定義して、
//    //それでcutしたほうがノイズを効率的に落とせるかも
//    if(abs(it->fT - tmin) > 1.2){
//      it = data->erase(it);
//      continue;
//    }
//    ++it;
//  }
//  //eraseしたので先頭イベントのイテレータを取り直す。(必須!!)
//  itmin = data->begin();
//  it = data->begin();
//  while(it != data->end()){
//    if(it->fX.Z() < itmin->fX.Z()) itmin = it;
//    //    if(it->fT < itmin->fT) itmin = it;
//    ++it;
//  }

//  //hit数に満たない場合はcut
//  if((int)data->size() < multihit){
//    return false;
//  }

  //2Dfit(zの誤差を無視する)でtracking
  Int_t point = 0;
  std::vector<double> pointx, pointy, pointz;
  std::vector<double> pointxe, pointye, pointze;
  it = data->begin();
  while(it != data->end()){
    pointx.push_back(it->fX.X());
    pointy.push_back(it->fX.Y());
    pointz.push_back(it->fX.Z());
    if(it->fId[0]%2){
      pointxe.push_back(width/sqrt(12));
      pointye.push_back(vl*treso/sqrt(2));
    }else{	 
      pointxe.push_back(vl*treso/sqrt(2));
      pointye.push_back(width/sqrt(12));
    }
    pointze.push_back(depth/sqrt(12));

    ++point;
    ++it;
  }
  double parFit[4];

  CAnalysis::LinearFit(pointz, pointx, pointxe, parFit[0], parFit[1]);
  CAnalysis::LinearFit(pointz, pointy, pointye, parFit[2], parFit[3]);
  double aminx = CAnalysis::SD(pointz, pointx, parFit[0], parFit[1]);
  double aminy = CAnalysis::SD(pointz, pointy, parFit[2], parFit[3]);
  double amin = sqrt(aminx*aminx+aminy*aminy);//rmsっぽいもの

  //  //rmsっぽいものを収集
  //  hamin->Fill(amin);

  // Simpleとイベント数が変わると面倒なので切り捨てはしない
  //
  //理想的には選択的にトラックを引くアルゴリズムが必要。
  //たとえばamin<5になるまで、計算に入れる点を変えるとか。
  //今はaminminで単純に切り捨てごめん。
//  if(amin > aminmin){
//    return false;
//  }

  //analysis of timing
  TVector3 xf(parFit[1], parFit[3], 1.);//fitした直線にそったベクトル
  double theta = xf.Angle(itmin->fX);
  double theta_rp = xf.Theta();
  double phi_rp = xf.Phi();

  // derive recoile proton energy from theta(lab)
  CParticle pin = CParticleDefinition::GetParticle("proton");
  CParticle nin = CParticleDefinition::GetParticle("neutron");
  CParticle pout = CParticleDefinition::GetParticle("proton");
  CParticle nout = CParticleDefinition::GetParticle("neutron");

  nin.k(ein-sp);//一陽子分離エネルギーを考慮。ほとんど効かないけど。
  pin.k(0);//Fermi motionは無視して静止していると考える。

  //lab系のthetaをパラメータとして、剛体の2体散乱をさせる。
  Double_t theta_cm = CKinema::Scatter(theta, 0., nin, pin, pout, nout);

  //pout,noutの角度を修正(上記のthetaはlab系でのneutronとの角度の差)
  pout.thetaphi(theta_rp, phi_rp);
  nout.thetaphibar(theta_rp, phi_rp); //??おかしい気がする(本当はdthetaだけ「回す」必要がある)。今は使って無いので無視

  //recoil protonの情報の保存
  //  dataexp->fPrp.SetPxPyPzE(pout.p(0), pout.p(1), pout.p(2), pout.e());

  //thetaを取得
  //  htheta_exp->Fill(theta);
  //  htheta_cm_exp->Fill(theta_cm);

  //timingの解析の本体
  double timing = 0;
  int count = 0;
  it = data->begin();
  while(it != data->end()){
    double tsum = -1.;
    if(it == itmin){
      tsum = 0;
      //先頭でhitしてから4層通過分程度時間がかかったeventは時間の計算には参加させない
    }else if(abs(it->fT - itmin->fT) < 1.0){
      double xtemp, ytemp, ztemp;
      if(it->fId[0]-itmin->fId[0] > 0){
	//二層め以降で成ったシグナルは荷電粒子によるものと思われ、
	//シミュレーション的にはシンチレータの端面で鳴るのでZを補正。
	//実験でも基本的に同じ補正でいいはずだが未知
	CAnalysis::line(it->fX.Z()-depth/2., parFit, xtemp, ytemp, ztemp);
      }else{
	CAnalysis::line(it->fX.Z(), parFit, xtemp, ytemp, ztemp);	
      }
      TVector3 X(xtemp, ytemp, ztemp);
      CAnalysis::line(itmin->fX.Z(), parFit, xtemp, ytemp, ztemp);
      TVector3 Xmin(xtemp, ytemp, ztemp);
      TVector3 distance_v = X - Xmin;
      double distance = distance_v.Mag();

      CParticle ptemp = pout;

      //検出器中を飛んだ時間の計算。先に物質中を飛んだ時間をだして、
      //あとから残りの空間を飛んだ時間を足す。
      //かなり荒いけど結構まともな近似
      //      tsum = CKinema::BetheBloch::TimeOut
      //	("p", "Scint", ptemp,
      //	 distance - (it->fId[0]-itmin->fId[0])*gap/cos(distance_v.Theta()), step);
      //      if(tsum >= 0.){
      //	//ほんとは2cm+2cm+2cm+....と交互に時間を足していかなければならない。
      //	//ここで少しmeanのずれが生じていると考えられる。
      //	tsum += (distance/2.)/((pout.beta()+ptemp.beta())*CKinema::fLightVelocity/2.);
      //      }

      //こっちの計算は真面目に計算した場合。エネルギーでのカットオフが必要
      double flightcos = cos(distance_v.Theta());
      int flightcount = 0;
      double flightdistance;

      tsum = 0;//初期値はtsum=-1なので0にしておく

      while(1){
	if((flightcount % 2) == 0){
	  if(flightcount == 0){
	    flightdistance = depth/2/flightcos;//一枚目は真ん中で発生したと考える
	  }else{
	    flightdistance = depth/flightcos;
	  }
	}else{
	  flightdistance = gap/flightcos;	    
	}

	if(distance < flightdistance) flightdistance = distance;

	if((flightcount % 2) == 0){
	  double tsumtemp;
	  tsumtemp = CKinema::BetheBloch::TimeOut("p", "Scint", ptemp, flightdistance, step);
	  //止まってしまった場合
	  if(tsumtemp < 0){
	    tsum = -1;
	    break;
	  }
	  tsum += tsumtemp;
	}else{
	  tsum += flightdistance/(ptemp.beta()*CKinema::fLightVelocity);
	}

	//ひょろひょろだったらその点を使った計算は不確定なのでいらない
	if(ptemp.k() < 20){
	  tsum = -1;
	  break;
	}

	distance -= flightdistance;
	if(distance <= 0) break;//発光点まで来たので終了

	++flightcount;
      }
    }
    if(tsum >= 0){
      //補正した結果、改善する方向に向かわない場合は補正に参加させない
      if(abs((it->fT - tsum) - itmin->fT) < treso*2){
	timing += it->fT - tsum;
	++count;
      }
    }
    ++it;
  }
  timing /= count;

  //位置はlinearfitで求まった先頭、時間は上の解析で補正した時間
  double xtemp, ytemp, ztemp;
  CAnalysis::line(itmin->fX.Z(),parFit,xtemp,ytemp,ztemp);
  //  dataexp->fX.SetXYZT(xtemp, ytemp, ztemp, timing);
  x = xtemp;
  y = ytemp;
  z = ztemp;
  t = timing;

  return true;
}

bool CAnalyzeNeutRCNP2n::Simple(std::deque<CDataHime::OneData> *data, double &x, double &y, double &z, double &t)
{
  std::deque<CDataHime::OneData>::iterator it;
  std::deque<CDataHime::OneData>::iterator itmin;

  if(data->empty()){
    return false;
  }

  itmin = data->begin();
  it = data->begin();
  while(it != data->end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  x = itmin->fX.X();
  y = itmin->fX.Y();
  z = itmin->fX.Z();
  t = itmin->fT;

  return true;
}

//trackが一つと思ってfit
double CAnalyzeNeutRCNP2n::OneTrackFit(std::deque<CDataHime::OneData> *data, double* parFit)
{
  if(data->size() < 3) return 1000;

  std::vector<double> pointx, pointy, pointz;
  std::vector<double> pointxe, pointye, pointze;
  std::deque<CDataHime::OneData>::iterator it;
  it = data->begin();
  while(it != data->end()){
    pointx.push_back(it->fX.X());
    pointy.push_back(it->fX.Y());
    pointz.push_back(it->fX.Z());
    if(it->fId[0]%2){
      pointxe.push_back(width/sqrt(12));
      pointye.push_back(vl*treso/sqrt(2));
    }else{	 
      pointxe.push_back(vl*treso/sqrt(2));
      pointye.push_back(width/sqrt(12));
    }
    pointze.push_back(depth/sqrt(12));

    ++it;
  }

  CAnalysis::LinearFit(pointz, pointx, pointxe, parFit[0], parFit[1]);
  CAnalysis::LinearFit(pointz, pointy, pointye, parFit[2], parFit[3]);
  double aminx = CAnalysis::SD(pointz, pointx, parFit[0], parFit[1]);
  double aminy = CAnalysis::SD(pointz, pointy, parFit[2], parFit[3]);

  return sqrt(aminx*aminx+aminy*aminy);
}

void CAnalyzeNeutRCNP2n::DivideTwo(int counter,
				   std::deque<CDataHime::OneData> *data,
				   std::deque<CDataHime::OneData> *data1,
				   std::deque<CDataHime::OneData> *data2)
{
  std::deque<CDataHime::OneData>::iterator it;
  it = data->begin();
  while(it != data->end()){
    int flag = counter % 2;
    counter /= 2;
    if(flag){
      data1->push_back(*it);
    }else{
      data2->push_back(*it);
    }
    ++it;
  }
}

//今は使ってない!!!--->TwoTrackFit
//
//counterに従って二つのグループに分け、それぞれをfit。
//counterは例えば5つの点を二つにわけるとき、
//1 2 3 4 5
//o x o o x
//とoとxにわけるとする。
//これをbitで表すことを考えれば、
//10110=22となる。
//つまり、2^5の数を順次渡せば総当たりでグループ分けができる。
double CAnalyzeNeutRCNP2n::TwoTrackFit(int counter, std::deque<CDataHime::OneData> *data, double* parFit1, double* parFit2)
{
  //aminが最小になるdataで二つに分別する
  std::deque<CDataHime::OneData> data1, data2;
  DivideTwo(counter, data, &data1, &data2);

  //  EliminateSolitaryPoint(&data1, 3);
  //  EliminateSolitaryPoint(&data2, 3);

  if((int)data1.size() < multihit || (int)data2.size() < multihit) return 1000;

  double amin1 = OneTrackFit(&data1, parFit1);
  double amin2 = OneTrackFit(&data2, parFit2);
  return sqrt(amin1*amin1+amin2*amin2);
}

CDataHime::OneData
CAnalyzeNeutRCNP2n::FrontData(std::deque<CDataHime::OneData> *data)
{
  std::deque<CDataHime::OneData>::iterator itmin = data->begin();
  std::deque<CDataHime::OneData>::iterator it = data->begin();
  while(it != data->end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }
  return *itmin;
}

CDataHime::OneData
CAnalyzeNeutRCNP2n::BackData(std::deque<CDataHime::OneData> *data)
{
  std::deque<CDataHime::OneData>::iterator itmin = data->begin();
  std::deque<CDataHime::OneData>::iterator it = data->begin();
  while(it != data->end()){
    if(itmin->fX.Z() < it->fX.Z()) itmin = it;
    ++it;
  }
  return *itmin;
}

bool CAnalyzeNeutRCNP2n::CrossTalk(std::deque<CDataHime::OneData> *data1,
				   std::deque<CDataHime::OneData> *data2)
{
  if(data1->size() + data2->size() > 9) return false; // 経験的な値(入射エネルギーによって変わる)
//  double dtheta = 0.2*0.5;
//  double dphi = 0.5*0.5;
  double dtheta = 0.2*1;
  double dphi = 0.5*1;
//  double dtheta = 0.2*2;
//  double dphi = 0.5*2;

  double theta1, phi1;
  NeutronAngle(data1, theta1, phi1);
  CDataHime::OneData onedata1 = FrontData(data1);
  double x1 = onedata1.fX.X();
  double y1 = onedata1.fX.Y();
  double z1 = onedata1.fX.Z();

  double theta2, phi2;
  NeutronAngle(data2, theta2, phi2);
  CDataHime::OneData onedata2 = FrontData(data2);
  double x2 = onedata2.fX.X();
  double y2 = onedata2.fX.Y();
  double z2 = onedata2.fX.Z();

#ifdef VISUALIZE
  //draw x1,y1,z1,theta1,phi1, line
  double tan_theta1 = tan(theta1);
  double parFit1[4] = {x1-z1*tan_theta1*cos(phi1), tan_theta1*cos(phi1),
		       y1-z1*tan_theta1*sin(phi1), tan_theta1*sin(phi1)};
  double tan_theta2 = tan(theta2);
  double parFit2[4] = {x2-z2*tan_theta2*cos(phi2), tan_theta2*cos(phi2),
		       y2-z2*tan_theta2*sin(phi2), tan_theta2*sin(phi2)};
  if(z2-z1>0){
    CAnalysis::DrawLine(parFit1, kBlack, z1, (z2-z1));
  }else{
    CAnalysis::DrawLine(parFit2, kBlack, z2, (z1-z2));
  }
#endif

  TVector3 direction21(x2-x1, y2-y1, z2-z1);
  double theta21 = direction21.Theta();
  double phi21 = direction21.Phi();

  TVector3 direction12(x1-x2, y1-y2, z1-z2);
  double theta12 = direction12.Theta();
  double phi12 = direction12.Phi();

  if(std::abs(theta21 - theta1) < dtheta &&
     std::abs(phi21 - phi1) < dphi){
    std::cout << "crossing1 " << theta21 << " " << phi21 << " " << theta1 << " " << phi1 << " " << theta2 << " " << phi2 << std::endl;
    
    return true;
  }

  if(std::abs(theta12 - theta2) < dtheta &&
     std::abs(phi12 - phi2) < dphi){
    std::cout << "crossing2 " << theta12 << " " << phi12 << " " << theta1 << " " << phi1 << " " << theta2 << " " << phi2 << std::endl;

    return true;
  }

  return false;
}

void CAnalyzeNeutRCNP2n::NeutronAngle(std::deque<CDataHime::OneData> *data, double &theta, double &phi)
{
  double parFit[4];
  OneTrackFit(data, parFit);

  std::deque<CDataHime::OneData>::iterator it;
  std::deque<CDataHime::OneData>::iterator itmin;

  //最速イベントではなく先頭イベント。
  itmin = data->begin();
  it = data->begin();
  while(it != data->end()){
    if(it->fX.Z() == itmin->fX.Z()){
      if(it->fT < itmin->fT) itmin = it;
    }else if(it->fX.Z() < itmin->fX.Z()){
      itmin = it;
    }
    ++it;
  }

  //analysis of timing
  TVector3 xf(parFit[1], parFit[3], 1.);//fitした直線にそったベクトル
  double theta_lab = xf.Angle(itmin->fX);
  double phi_lab = xf.Phi();
  double dtheta = theta_lab - xf.Theta();

  // derive recoile proton energy from theta(lab)
  CParticle pin = CParticleDefinition::GetParticle("proton");
  CParticle nin = CParticleDefinition::GetParticle("neutron");
  CParticle pout = CParticleDefinition::GetParticle("proton");
  CParticle nout = CParticleDefinition::GetParticle("neutron");

  //cal ein by simple method
  TVector3 beta = 1./(itmin->fT*CAnalyzeTool::c)*itmin->fX;
  TVector3 momentum = nin.m()*1./sqrt(1.-beta.Mag()*beta.Mag())*beta;

  nin.p(momentum.Px(), momentum.Py(), momentum.Pz());
  nin.k(nin.k()-sp); //一陽子分離エネルギーを考慮。ほとんど効かないけど。
  pin.k(0);//Fermi motionは無視して静止していると考える。

  //lab系のthetaをパラメータとして、剛体の2体散乱をさせる。
  CKinema::Scatter(theta_lab, phi_lab, nin, pin, pout, nout);

  //pout,noutの角度を修正(上記のthetaはlab系でのneutronとの角度の差)
  //本当はdthetaだけ回転させるべき。
  //  theta = nout.theta() + dtheta;??
  theta = nout.theta();
  phi = nout.phi();
}

//孤立点を取り除く
void CAnalyzeNeutRCNP2n::EliminateSolitaryPoint(std::deque<CDataHime::OneData> *data, double distance_scale)
{
  std::deque<CDataHime::OneData>::iterator it;
  it = data->begin();
  while(it != data->end()){
    //    std::cout << "\n next " << std::flush;
    bool near = false; //近傍に点が一つでもいるか判定
    std::deque<CDataHime::OneData>::iterator it_inner;
    it_inner = data->begin();
    while(it_inner != data->end()){
      if(it == it_inner){
	++it_inner;
	continue;
      }
      double dx = (it->fX.X()-it_inner->fX.X())/4; // 割る6は近傍とみなす距離で規格化
      double dy = (it->fX.Y()-it_inner->fX.Y())/4;
      double dz = (it->fX.Z()-it_inner->fX.Z())/4;
      double distance = sqrt((dx*dx+dy*dy+dz*dz)/3);
      //      std::cout << distance << " " << std::flush;
      if(distance < distance_scale) near = true;
      ++it_inner;
    }
    if(!near){
      //      std::cout << "\n " << it->fX.X() << " " << it->fX.Y() << " " << it->fX.Z() << std::endl;
      it = data->erase(it);
    }else{
      ++it;
    }
  }
}

void CAnalyzeNeutRCNP2n::Grouping(std::deque<CDataHime::OneData> *data,
				  std::deque<std::deque<CDataHime::OneData> > *data_array)
{
  std::deque<CDataHime::OneData> data_copy(*data);

  std::deque<CDataHime::OneData>::iterator it_newgrouphead;
  it_newgrouphead = data_copy.begin();
  while(it_newgrouphead != data_copy.end()){

    //既にdata_arrayの中にいないか探索
    bool found = false;
    std::deque<std::deque<CDataHime::OneData> >::iterator it_data_array = data_array->begin();
    while(it_data_array != data_array->end()){
      if(std::find(it_data_array->begin(), it_data_array->end(), *it_newgrouphead)
	 != it_data_array->end()){
	found = true;
      }
      ++it_data_array;
    }
    if(found){
      it_newgrouphead = data_copy.erase(it_newgrouphead);
      continue;
    }

    //新しいhakoにgroupとなるdataを詰めていく
    std::deque<CDataHime::OneData> data_hako;
    data_hako.push_back(*it_newgrouphead);

    std::deque<CDataHime::OneData>::iterator it_hako;
    it_hako = data_hako.begin();
    while(it_hako != data_hako.end()){
      std::deque<CDataHime::OneData>::iterator it_inner;
      it_inner = data_copy.begin();
      while(it_inner != data_copy.end()){
	if(std::find(data_hako.begin(), data_hako.end(), *it_inner) != data_hako.end()){ //既にhakoの中にいないか探索
	  ++it_inner;
	  continue;
	}
	
	double dx = (it_hako->fX.X()-it_inner->fX.X())/4; // 割る6は近傍とみなす距離で規格化
	double dy = (it_hako->fX.Y()-it_inner->fX.Y())/4;
	double dz = (it_hako->fX.Z()-it_inner->fX.Z())/4;
	double distance = sqrt((dx*dx+dy*dy+dz*dz)/3);
	if(distance < 1.8) data_hako.push_back(*it_inner);
	++it_inner;
      }
      ++it_hako;
    }
    data_array->push_back(data_hako);
    it_newgrouphead = data_copy.erase(it_newgrouphead);
  }
}
